@@ -147,24 +147,63 @@ class ArrayField(IndexedFieldMixin, Field):
     default_index_type = 'GIN'
     passthrough = True
 
-    def __init__(self, field_class=IntegerField, dimensions=1, *args,
-                 **kwargs):
-        self.__field = field_class(*args, **kwargs)
+    def __init__(self, field_class=IntegerField, field_kwargs=None,
+                 dimensions=1, convert_values=False, *args, **kwargs):
+        self.__field = field_class(**(field_kwargs or {}))
         self.dimensions = dimensions
+        self.convert_values = convert_values
         self.field_type = self.__field.field_type
         super(ArrayField, self).__init__(*args, **kwargs)
 
+    def bind(self, model, name, set_attribute=True):
+        ret = super(ArrayField, self).bind(model, name, set_attribute)
+        self.__field.bind(model, '__array_%s' % name, False)
+        return ret
+
     def ddl_datatype(self, ctx):
         data_type = self.__field.ddl_datatype(ctx)
         return NodeList((data_type, SQL('[]' * self.dimensions)), glue='')
 
     def db_value(self, value):
-        if value is not None:
-            return list(value) if not isinstance(value, Node) else value
+        if value is None or isinstance(value, Node):
+            return value
+        elif self.convert_values:
+            return self._process(self.__field.db_value, value, self.dimensions)
+        else:
+            return value if isinstance(value, list) else list(value)
+
+    def python_value(self, value):
+        if self.convert_values and value is not None:
+            conv = self.__field.python_value
+            if isinstance(value, list):
+                return self._process(conv, value, self.dimensions)
+            else:
+                return conv(value)
+        else:
+            return value
+
+    def _process(self, conv, value, dimensions):
+        dimensions -= 1
+        if dimensions == 0:
+            return [conv(v) for v in value]
+        else:
+            return [self._process(conv, v, dimensions) for v in value]
 
     def __getitem__(self, value):
         return ObjectSlice.create(self, value)
 
+    def _e(op):
+        def inner(self, rhs):
+            return Expression(self, op, ArrayValue(self, rhs))
+        return inner
+    __eq__ = _e(OP.EQ)
+    __ne__ = _e(OP.NE)
+    __gt__ = _e(OP.GT)
+    __ge__ = _e(OP.GTE)
+    __lt__ = _e(OP.LT)
+    __le__ = _e(OP.LTE)
+    __hash__ = Field.__hash__
+
     def contains(self, *items):
         return Expression(self, ACONTAINS, ArrayValue(self, items))
 
@@ -190,6 +229,7 @@ class DateTimeTZField(DateTimeField):
 
 class HStoreField(IndexedFieldMixin, Field):
     field_type = 'HSTORE'
+    __hash__ = Field.__hash__
 
     def __getitem__(self, key):
         return Expression(self, HKEY, Value(key))
@@ -262,6 +302,7 @@ def cast_jsonb(node):
 class BinaryJSONField(IndexedFieldMixin, JSONField):
     field_type = 'JSONB'
     default_index_type = 'GIN'
+    __hash__ = Field.__hash__
 
     def contains(self, other):
         if isinstance(other, (list, dict)):
@@ -287,6 +328,7 @@ class BinaryJSONField(IndexedFieldMixin, JSONField):
 class TSVectorField(IndexedFieldMixin, TextField):
     field_type = 'TSVECTOR'
     default_index_type = 'GIN'
+    __hash__ = Field.__hash__
 
     def match(self, query, language=None):
         params = (language, query) if language is not None else (query,)
@@ -326,7 +368,7 @@ class FetchManyCursor(object):
         while True:
             rows = self.cursor.fetchmany(self.array_size)
             if not rows:
-                raise StopIteration
+                return
             for row in rows:
                 yield row
 
@@ -391,7 +433,7 @@ class PostgresqlExtDatabase(PostgresqlDatabase):
             register_hstore(conn, globally=True)
         return conn
 
-    def cursor(self, commit):
+    def cursor(self, commit=None):
         if self.is_closed():
             self.connect()
         if commit is __named_cursor__:
