@@ -3,14 +3,17 @@ try:
 except ImportError:
     OrderedDict = dict
 from collections import namedtuple
+from inspect import isclass
 import re
 
 from peewee import *
+from peewee import _StringField
+from peewee import text_type
 try:
-    from MySQLdb.constants import FIELD_TYPE
+    from pymysql.constants import FIELD_TYPE
 except ImportError:
     try:
-        from pymysql.constants import FIELD_TYPE
+        from MySQLdb.constants import FIELD_TYPE
     except ImportError:
         FIELD_TYPE = None
 try:
@@ -38,7 +41,7 @@ class Column(object):
 
     def __init__(self, name, field_class, raw_column_type, nullable,
                  primary_key=False, column_name=None, index=False,
-                 unique=False):
+                 unique=False, default=None, extra_parameters=None):
         self.name = name
         self.field_class = field_class
         self.raw_column_type = raw_column_type
@@ -47,6 +50,8 @@ class Column(object):
         self.column_name = column_name
         self.index = index
         self.unique = unique
+        self.default = default
+        self.extra_parameters = extra_parameters
 
         # Foreign key metadata.
         self.rel_model = None
@@ -67,14 +72,18 @@ class Column(object):
 
     def get_field_parameters(self):
         params = {}
+        if self.extra_parameters is not None:
+            params.update(self.extra_parameters)
 
         # Set up default attributes.
         if self.nullable:
             params['null'] = True
         if self.field_class is ForeignKeyField or self.name != self.column_name:
             params['column_name'] = "'%s'" % self.column_name
-        if self.primary_key and self.field_class is not AutoField:
+        if self.primary_key and not issubclass(self.field_class, AutoField):
             params['primary_key'] = True
+        if self.default is not None:
+            params['constraints'] = '[SQL("DEFAULT %s")]' % self.default
 
         # Handle ForeignKeyField-specific attributes.
         if self.is_foreign_key():
@@ -116,7 +125,12 @@ class Column(object):
 
     def get_field(self):
         # Generate the field definition for this column.
-        field_params = self.get_field_parameters()
+        field_params = {}
+        for key, value in self.get_field_parameters().items():
+            if isclass(value) and issubclass(value, Field):
+                value = value.__name__
+            field_params[key] = value
+
         param_str = ', '.join('%s=%s' % (k, v)
                               for k, v in sorted(field_params.items()))
         field = '%s = %s(%s)' % (
@@ -147,7 +161,7 @@ class Metadata(object):
             for metadata in self.database.get_columns(table, schema))
 
         # Look up the actual column type for each column.
-        column_types = self.get_column_types(table, schema)
+        column_types, extra_params = self.get_column_types(table, schema)
 
         # Look up the primary keys.
         pk_names = self.get_primary_keys(table, schema)
@@ -155,22 +169,38 @@ class Metadata(object):
             pk = pk_names[0]
             if column_types[pk] is IntegerField:
                 column_types[pk] = AutoField
+            elif column_types[pk] is BigIntegerField:
+                column_types[pk] = BigAutoField
 
         columns = OrderedDict()
         for name, column_data in metadata.items():
+            field_class = column_types[name]
+            default = self._clean_default(field_class, column_data.default)
+
             columns[name] = Column(
                 name,
-                field_class=column_types[name],
+                field_class=field_class,
                 raw_column_type=column_data.data_type,
                 nullable=column_data.null,
                 primary_key=column_data.primary_key,
-                column_name=name)
+                column_name=name,
+                default=default,
+                extra_parameters=extra_params.get(name))
 
         return columns
 
     def get_column_types(self, table, schema=None):
         raise NotImplementedError
 
+    def _clean_default(self, field_class, default):
+        if default is None or field_class in (AutoField, BigAutoField) or \
+           default.lower() == 'null':
+            return
+        if issubclass(field_class, _StringField) and \
+           isinstance(default, text_type) and not default.startswith("'"):
+            default = "'%s'" % default
+        return default or "''"
+
     def get_foreign_keys(self, table, schema=None):
         return self.database.get_foreign_keys(table, schema)
 
@@ -201,6 +231,19 @@ class PostgresqlMetadata(Metadata):
         1700: DecimalField,
         2950: TextField, # UUID
     }
+    array_types = {
+        1000: BooleanField,
+        1001: BlobField,
+        1005: SmallIntegerField,
+        1007: IntegerField,
+        1009: TextField,
+        1014: CharField,
+        1015: CharField,
+        1016: BigIntegerField,
+        1115: DateTimeField,
+        1182: DateField,
+        1183: TimeField,
+    }
     extension_import = 'from playhouse.postgres_ext import *'
 
     def __init__(self, database):
@@ -208,20 +251,29 @@ class PostgresqlMetadata(Metadata):
 
         if postgres_ext is not None:
             # Attempt to add types like HStore and JSON.
-            cursor = self.execute('select oid, typname from pg_type;')
+            cursor = self.execute('select oid, typname, format_type(oid, NULL)'
+                                  ' from pg_type;')
             results = cursor.fetchall()
 
-            for oid, typname in results:
-                if 'json' in typname:
+            for oid, typname, formatted_type in results:
+                if typname == 'json':
                     self.column_map[oid] = postgres_ext.JSONField
-                elif 'hstore' in typname:
+                elif typname == 'jsonb':
+                    self.column_map[oid] = postgres_ext.BinaryJSONField
+                elif typname == 'hstore':
                     self.column_map[oid] = postgres_ext.HStoreField
-                elif 'tsvector' in typname:
+                elif typname == 'tsvector':
                     self.column_map[oid] = postgres_ext.TSVectorField
 
+            for oid in self.array_types:
+                self.column_map[oid] = postgres_ext.ArrayField
+
     def get_column_types(self, table, schema):
         column_types = {}
+        extra_params = {}
         extension_types = set((
+            postgres_ext.ArrayField,
+            postgres_ext.BinaryJSONField,
             postgres_ext.JSONField,
             postgres_ext.TSVectorField,
             postgres_ext.HStoreField)) if postgres_ext is not None else set()
@@ -232,13 +284,15 @@ class PostgresqlMetadata(Metadata):
 
         # Store column metadata in dictionary keyed by column name.
         for column_description in cursor.description:
-            column_types[column_description.name] = self.column_map.get(
-                column_description.type_code,
-                UnknownField)
-            if column_types[column_description.name] in extension_types:
+            name = column_description.name
+            oid = column_description.type_code
+            column_types[name] = self.column_map.get(oid, UnknownField)
+            if column_types[name] in extension_types:
                 self.requires_extension = True
+            if oid in self.array_types:
+                extra_params[name] = {'field_class': self.array_types[oid]}
 
-        return column_types
+        return column_types, extra_params
 
     def get_columns(self, table, schema=None):
         schema = schema or 'public'
@@ -300,7 +354,7 @@ class MySQLMetadata(Metadata):
             name, type_code = column_description[:2]
             column_types[name] = self.column_map.get(type_code, UnknownField)
 
-        return column_types
+        return column_types, {}
 
 
 class SqliteMetadata(Metadata):
@@ -352,7 +406,7 @@ class SqliteMetadata(Metadata):
         for column in columns:
             column_types[column.name] = self._map_col(column.data_type)
 
-        return column_types
+        return column_types, {}
 
 
 _DatabaseMetadata = namedtuple('_DatabaseMetadata', (
@@ -398,8 +452,10 @@ class Introspector(object):
             metadata = PostgresqlMetadata(database)
         elif isinstance(database, MySQLDatabase):
             metadata = MySQLMetadata(database)
-        else:
+        elif isinstance(database, SqliteDatabase):
             metadata = SqliteMetadata(database)
+        else:
+            raise ValueError('Introspection not supported for %r' % database)
         return cls(metadata, schema=schema)
 
     def get_database_class(self):
@@ -432,15 +488,17 @@ class Introspector(object):
             column = '_' + column
         return column
 
-    def introspect(self, table_names=None, literal_column_names=False):
+    def introspect(self, table_names=None, literal_column_names=False,
+                   include_views=False):
         # Retrieve all the tables in the database.
-        if self.schema:
-            tables = self.metadata.database.get_tables(schema=self.schema)
-        else:
-            tables = self.metadata.database.get_tables()
+        tables = self.metadata.database.get_tables(schema=self.schema)
+        if include_views:
+            views = self.metadata.database.get_views(schema=self.schema)
+            tables.extend([view.name for view in views])
 
         if table_names is not None:
             tables = [table for table in tables if table in table_names]
+        table_set = set(tables)
 
         # Store a mapping of table name -> dictionary of columns.
         columns = {}
@@ -467,6 +525,14 @@ class Introspector(object):
             except ValueError as exc:
                 err(*exc.args)
                 foreign_keys[table] = []
+            else:
+                # If there is a possibility we could exclude a dependent table,
+                # ensure that we introspect it so FKs will work.
+                if table_names is not None:
+                    for foreign_key in foreign_keys[table]:
+                        if foreign_key.dest_table not in table_set:
+                            tables.append(foreign_key.dest_table)
+                            table_set.add(foreign_key.dest_table)
 
             model_names[table] = self.make_model_name(table)
 
@@ -544,14 +610,16 @@ class Introspector(object):
             indexes)
 
     def generate_models(self, skip_invalid=False, table_names=None,
-                        literal_column_names=False, bare_fields=False):
-        database = self.introspect(table_names=table_names,
-                                   literal_column_names=literal_column_names)
+                        literal_column_names=False, bare_fields=False,
+                        include_views=False):
+        database = self.introspect(table_names, literal_column_names,
+                                   include_views)
         models = {}
 
         class BaseModel(Model):
             class Meta:
                 database = self.metadata.database
+                schema = self.schema
 
         def _create_model(table, models):
             for foreign_key in database.foreign_keys[table]:
@@ -611,7 +679,13 @@ class Introspector(object):
 
                     # Generate a unique related name.
                     params['backref'] = '%s_%s_rel' % (table, column_name)
-                if column_name in column_indexes and not column.is_primary_key():
+
+                if column.default is not None:
+                    constraint = SQL('DEFAULT %s' % column.default)
+                    params['constraints'] = [constraint]
+
+                if column_name in column_indexes and not \
+                   column.is_primary_key():
                     if column_indexes[column_name]:
                         params['unique'] = True
                     elif not column.is_foreign_key():
