@@ -5,7 +5,7 @@ Project page: https://github.com/leapcode/pysqlcipher/
 
 **WARNING!!! EXPERIMENTAL!!!**
 
-* Although this extention's code is short, it has not been propery
+* Although this extention's code is short, it has not been properly
   peer-reviewed yet and may have introduced vulnerabilities.
 * The code contains minimum values for `passphrase` length and
   `kdf_iter`, as well as a default value for the later.
@@ -25,7 +25,7 @@ Example usage:
 * `passphrase`: should be "long enough".
   Note that *length beats vocabulary* (much exponential), and even
   a lowercase-only passphrase like easytorememberyethardforotherstoguess
-  packs more noise than 8 random printable chatacters and *can* be memorized.
+  packs more noise than 8 random printable characters and *can* be memorized.
 * `kdf_iter`: Should be "as much as the weakest target machine can afford".
 
 When opening an existing database, passphrase and kdf_iter should be identical
@@ -52,16 +52,14 @@ https://gist.github.com/thedod/11048875
 """
 import datetime
 import decimal
+import sys
 
 from peewee import *
 from playhouse.sqlite_ext import SqliteExtDatabase
-try:
+if sys.version_info[0] != 3:
     from pysqlcipher import dbapi2 as sqlcipher
-except ImportError:
-    try:
-        from pysqlcipher3 import dbapi2 as sqlcipher
-    except ImportError:
-        raise ImportError('Sqlcipher python bindings not found.')
+else:
+    from pysqlcipher3 import dbapi2 as sqlcipher
 
 sqlcipher.register_adapter(decimal.Decimal, str)
 sqlcipher.register_adapter(datetime.date, str)
@@ -86,16 +84,38 @@ class _SqlCipherDatabase(object):
         conn = sqlcipher.connect(self.database, **params)
         conn.isolation_level = None
         try:
-            self._add_conn_hooks(conn)
             conn.execute(
                 'PRAGMA key=\'{0}\''.format(passphrase.replace("'", "''")))
             conn.execute('PRAGMA kdf_iter={0:d}'.format(kdf_iter))
+            self._add_conn_hooks(conn)
         except:
             conn.close()
             raise
         else:
             return conn
 
+    def set_passphrase(self, passphrase):
+        if not self.is_closed():
+            raise ImproperlyConfigured('Cannot set passphrase when database '
+                                       'is open. To change passphrase of an '
+                                       'open database use the rekey() method.')
+
+        self.connect_params['passphrase'] = passphrase
+
+    def rekey(self, passphrase):
+        if len(passphrase) < 8:
+            raise ImproperlyConfigured(
+                'SqlCipherDatabase passphrase should be at least eight '
+                'character long.')
+
+        if self.is_closed():
+            self.connect()
+
+        self.execute_sql(
+            'PRAGMA rekey=\'{0}\''.format(passphrase.replace("'", "''")))
+        self.connect_params['passphrase'] = passphrase
+        return True
+
 
 class SqlCipherDatabase(_SqlCipherDatabase, SqliteDatabase):
     pass
